#!/usr/bin/perl



package Uird;

use strict;
use Socket;
use IO::Socket;

sub new {
  my( $PKG, %opts) = @_;

  my $self = {};
  bless $self, $PKG;

  my $PORT = $opts{'port'} || 5253;
  $self->{'debug'} = $opts{'debug'};
  $self->{'callback'} = $opts{'callback'} || sub { print "Uird.pm: Data received: ".join( "-", @_)."\n"; };
  $self->{'password'} = substr( $opts{'password'}, 0, 8) . ("\0" x (9-length( $opts{'password'})));

  $self->{'socket'} = new IO::Socket;
  socket( $self->{'socket'}, PF_INET, SOCK_DGRAM, getprotobyname( 'udp')) || die "Uird: Couldn't create socket: $!\n";
  my $mypaddr = sockaddr_in( $PORT, inet_aton( "0.0.0.0"));  # Bind to first available port on default interface
  bind( $self->{'socket'}, $mypaddr) || die "Uird: Couldn't bind: $!\n";

  return( $self);
}

sub install_callback {
  my( $self, $callback) = @_;
  $self->{'callback'} = $callback;
}

sub data_as_charstr {
  my( $pkg, @data) = @_;
  my $ret;
  foreach my $k (@data) {
    $ret.=chr( $k);
  }
  return( $ret);
}

sub do {
  my( $self, $time_out) = @_;
  my( $rin, $selret, $msg);

  ## SELECT
  vec( $rin, fileno( $self->{'socket'}), 1) = 1;
  $selret = select( $rin, undef, undef, $time_out);

  ## IF NOT TIMEOUT
  if( $selret > 0) {
    recv( $self->{'socket'}, $msg, 30, 0);

    ## Unpack msg-struct
    my( $msg_magic, $msg_type, $msg_password, @msg_data) = unpack( "a5Ca9CCCCCC", $msg);

    ## Check magic cookie
    if( $msg_magic ne "UIRD\0") {
      print STDERR "Uird.pm: Error magic value received\n";
    } else {
      ## Check password
      if( $msg_password ne $self->{'password'}) {
        print STDERR "Uird.pm: Wrong password received - ignoring packet, msg_password='$msg_password'\n";
      } else {
        if( $msg_type == 0) {		## Init
          print STDERR "Uird.pm: UIRD-module initialized and ready\n";
        } elsif( $msg_type == 1) {	## Error
          print STDERR "Uird.pm: UIRD-module encountered an error\n";
        } elsif( $msg_type == 2) {	## Data received
          &{$self->{'callback'}}( @msg_data);
        } elsif( $msg_type == 3) {	## UIRD exiting
          print STDERR "Uird.pm: uird is exiting\n";
        } else {			## Unknown
          print STDERR "Uird.pm: Unknown packet-type received, msgtype=".$msg_type."\n";
        }
      }
    }
  }
}


1;


