/*
 *  handler_list.c
 *  Reads the configuration-file
 *  Copyright (C) 2000  Carl Bingel, bingel@dacom.se
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include <stdio.h>
#include <errno.h>

#include "uird.h"

#define LINEBUFSIZE 150

struct handler_list *insert_handler( struct handler_list *hlist, char *irdata, char *command);

struct handlers *read_handler_list( char *filename) {
  FILE *fh;
  char linebuf[ LINEBUFSIZE];
  struct handlers *ret;
  struct handler_list *h_default, *h_normal;
  unsigned int i[ 6], linecounter, k;
  char irdata_buf[ 6];

  ret = (struct handlers *)malloc( sizeof( struct handlers));
  ret->h_default = ret->h_always = ret->h_normal = NULL;

  fh = fopen( filename, "r");
  if (fh == NULL) {
    fprintf( stderr, "uird: opening config-file '%s': %s\n", filename, strerror( errno));
    exit( -3);
  }

  linecounter = 0;
  while( fgets( linebuf, LINEBUFSIZE, fh) != NULL) {
    linecounter++;
    linebuf[ strlen( linebuf)-1] = '\0';	/* Chop LF */
    if ( strncmp( linebuf, "default:", 8) == 0) 			// Default action 
      ret->h_default = insert_handler( NULL, " TOMT ", linebuf+9);
    else if ( strncmp( linebuf, "always:", 7) == 0) 			// Always execute this
      ret->h_always = insert_handler( NULL, " TOMT ", linebuf+8);
    else if ( (linebuf[ 0] == '#') || (linebuf[ 0] == '\0')) {		// Comment or blank line
      // Void
    } else {								// Normal line
      if ( sscanf( linebuf, "%u-%u-%u-%u-%u-%u: ", i, i+1, i+2, i+3, i+4, i+5) != 6) 
        fprintf( stderr, "%s: error config line %u: %s\n", filename, linecounter, linebuf);
      else {
        for( k=0; k<6; k++) 
          irdata_buf[ k] = i[k];
        k = 0;
        while( linebuf[ k++] != ':');
        ret->h_normal = insert_handler( ret->h_normal, irdata_buf, linebuf + k);
      }
    }
  }

  fclose( fh);
  
  return( ret);
}


/*  Inserts handler in handler-list  */
struct handler_list *insert_handler( struct handler_list *hlist, char *irdata, char *command) {
  unsigned int k;
  char t[100];
  struct handler_list *new;

  // Cut heading whitespaces 
  while( isspace( command[ 0])) 
    command++;

  // Allocate new record
  if ( (new = (struct handler_list *)malloc( sizeof( struct handler_list))) == NULL) {
    fprintf( stderr, "uird: malloc: out of memory\n");
    exit( -4);
  }

  new->next = hlist;
  new->command = NULL;
  strncpy( new->datastring, irdata, 6);
  if ( strncmp( command, "PERL_HANDLER", 12) == 0) {
    if ( sscanf( command+13, "%s", t) == 1) {
      new->operation = 1;
      new->command = (char *)strdup( t);
    } 
  } else {
    new->operation = 0;
    new->command = (char *)strdup( command);
  }
  return( new);
}

