/*
 *  init_irmodule.c
 *  Opens tty and initializes irmodule
 *  Copyright (C) 2000  Carl Bingel, bingel@dacom.se
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <termios.h>
#include <string.h>


/*  Initialises the IR-module and returns filehandle to the open TTY  */
int init_ir_module( char *tty_name) {
  int fd;
  char chbuf[ 3];
  struct termios tty_attrs;

  memset( chbuf, 0, 3);

  fd = open( tty_name, O_RDWR | O_NOCTTY | O_NDELAY);
  if ( fd < 0) {
    fprintf( stderr, "uird: open_port: Unable to open %s - %s\n", tty_name, strerror( errno));
    exit( -1);
  }

  fcntl( fd, F_SETFL, FNDELAY);		/* Set FNDELAY = Don't block on read */

  tcgetattr( fd, &tty_attrs);		/* Fetch termios attributes struct */
  cfsetispeed( &tty_attrs, B9600);		/* Set baudrate to 9600 */
  cfsetospeed( &tty_attrs, B9600);
  tty_attrs.c_cflag |= (CLOCAL | CREAD);	/* Set CLOCAL = Local line, CREAD = Enable receiver */
  tty_attrs.c_cflag &= ~PARENB;			/* 8 bits, no parity */
  tty_attrs.c_cflag &= ~CSTOPB;
  tty_attrs.c_cflag &= ~CSIZE;
  tty_attrs.c_cflag |= CS8;                           
  tty_attrs.c_cflag &= ~CRTSCTS;		/* Disable RTS/CTS */
  tty_attrs.c_lflag &= ~(ICANON | ECHO | ISIG);	/* Disable Canonical input, Echo */
  tcsetattr( fd, TCSANOW, &tty_attrs);

  /*  Wait for half-a-sec for the IR-module to stabilize  */
  usleep( 500000);

  tcsetattr( fd, TCSAFLUSH, &tty_attrs);	/* Flush input and output fifos */

  /*  Send the "IR" string for init of the IR-module  */
  write( fd, "I", 1);
  usleep( 1000);	/* Wait 1ms */
  write( fd, "R", 1);
  usleep( 10000);	/* Wait 10ms */
  read( fd, chbuf, 2);
  if ( strcmp( chbuf, "OK") != 0) {
    printf( "Couldn't initialize IR-module. Module did not return 'OK'. Module returned '%s'.\n", chbuf);
    exit( -2);
  }

  tcsetattr( fd, TCSAFLUSH, &tty_attrs);	/* Flush input and output fifos */

  return( fd);
}





