/*
 *  opts.c
 *  Parses the command line options
 *  Copyright (C) 2000, 2001  Carl Bingel, bingel@dacom.se
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include <stdio.h>

#include "uird.h"
#include "defaults.h"

// Parse commandline options and fill in options-struct
//
void parse_cmdline_args( int argc, char **argv, struct options_struct *options) {
  int k;

  // Defaults
  options->config_file_name = DFLT_CONFIG_FILE_NAME;
  options->tty_name = DFLT_TTY_NAME;
  options->operation_mode = 0;
  options->perl_file_name = DFLT_PERL_FILE_NAME;
  options->perl_handler_name = DFLT_PERL_HANDLER_NAME;
  options->daemon_mode = DFLT_DAEMON_MODE;
  options->udp_dst_addr = DFLT_UDP_ADDR;
  options->udp_dst_port = DFLT_UDP_PORT;
  options->udp_passwd = "";

  // Parse
  for( k=1; k<argc; k++) {
    if ( strcmp( argv[ k], "-f") == 0) {                // -f <file>    Specify config-file
      options->config_file_name = argv[ ++k];
    } else if( strcmp( argv[ k], "-t") == 0) {          // -t <tty>     Specify which TTY the ir-module uses
      options->tty_name = argv[ ++k];
    } else if( strcmp( argv[ k], "-d") == 0) {          // -d           Daemon-mode (fork and exit)
      options->daemon_mode = 1;
    } else if( strcmp( argv[ k], "-N") == 0) {          // -N           "Normal operation"
      options->operation_mode = 0;
    } else if( strcmp( argv[ k], "-C") == 0) {          // -C           Code-testing-mode
      options->operation_mode = 1;
    } else if( strcmp( argv[ k], "-P") == 0) {          // -P           Perl-mode
      options->operation_mode = 2;
    } else if( strcmp( argv[ k], "-L") == 0) {          // -L           Learning-mode
      options->operation_mode = 3;
    } else if( strcmp( argv[ k], "-h") == 0) {          // -h           Print help
      options->operation_mode = 4;
    } else if( strcmp( argv[ k], "-K") == 0) {          // -K           Kill running uird in daemon mode
      options->operation_mode = 5;
    } else if( strcmp( argv[ k], "-U") == 0) {          // -U           Network-mode
      options->operation_mode = 6;
    } else if( strcmp( argv[ k], "-p") == 0) {          // -p <file>    Which perl-file to load
      options->perl_file_name = argv[ ++k];
    } else if( strcmp( argv[ k], "-s") == 0) {          // -s <handlername> Which perl-handler to call in "Perl-mode"
      options->perl_handler_name = argv[ ++k];
    } else if( strcmp( argv[ k], "-Uaddr") == 0) {      // -Uaddr <addr> Specify ip-address to send data to
      options->udp_dst_addr = argv[ ++k];
    } else if( strcmp( argv[ k], "-Uport") == 0) {      // -Uport <port> Specify port to send data to
      options->udp_dst_port = atoi( argv[ ++k]);
    } else if( strcmp( argv[ k], "-Upw") == 0) {        // -Upw <passwd> Specify password to use in network-mode
      options->udp_passwd = argv[ ++k];
    } else {
      fprintf( stderr, "uird: invalid option '%s'\nTry 'uird -h' for more information.\n", argv[ k]);
      exit( -6);
    }
  }
}



